# %% [markdown]
# 1、导入库pandas、numpy、warnings

# %%
import pandas as pd
import numpy as np

# %% [markdown]
# 2、读取数据,查看除表头外的前5行数据

# %%
df=pd.read_csv('E:\\data\\03\\credit-default.csv')
df.head()

# %% [markdown]
# 3、查看目标变量’Target’列中的数据分布状况

# %%
df['Target'].value_counts()#方法一

# %%
import seaborn as sn  #方法二
sn.countplot(df['Target'])

# %% [markdown]
# 4、使用corr()方法获取spearman相关性系数矩阵并输出；

# %%
df_corr=df.corr()
df_corr

# %% [markdown]
# 5、对相关性超过0.8的属性进行删除；

# %%
list_drop=[]
for x in df_corr.columns:
    for y in df_corr.index:
        c=df_corr.loc[x,y]
        if c>0.8 and x!=y:
            list_drop.append(x)

# %%
for x in list_drop:
    del df[x]

# %%
df.shape

# %% [markdown]
#  6、使用heatmap绘制相关性矩阵热力图；

# %%
import matplotlib.pyplot as plt
plt.figure(figsize=(15,12))
sn.heatmap(df_corr)

# %% [markdown]
# 7、对数值型变量’ House_State’ 、’ Year_Income’、’ Couple_Year_Income’、’  L12_Month_Pay_Amount’中的缺失值进行填充；
# 

# %%
df['House_State']=df['House_State'].replace(np.nan,df['House_State'].mode()[0])
df['Year_Income']=df['Year_Income'].replace(np.nan,df['Year_Income'].mean())
df['Couple_Year_Income']=df['Couple_Year_Income'].replace(np.nan,df['Couple_Year_Income'].mean())
df['L12_Month_Pay_Amount']=df['L12_Month_Pay_Amount'].replace(np.nan,df['L12_Month_Pay_Amount'].mean())

# %% [markdown]
# 8、对名义型变量(字符串类型的数据，DataFrame中表示的是object型) ’
#   Gender’ 中的缺失值进行填充；
# 

# %%

df['Unit_Kind']=df['Unit_Kind'].replace('Be',df['Unit_Kind'].mode()[0])
df['Unit_Kind']=df['Unit_Kind'].replace('ae',df['Unit_Kind'].mode()[0])
df['Unit_Kind']=df['Unit_Kind'].replace('Da',df['Unit_Kind'].mode()[0])
df['Occupation']=df['Occupation'].replace('Y0000',df['Occupation'].mode()[0])
df['Occupation']=df['Occupation'].replace('X0000',df['Occupation'].mode()[0])
df['Occupation']=df['Occupation'].replace(' ',df['Occupation'].mode()[0])
df['Occupation']=df['Occupation'].replace('   ',df['Occupation'].mode()[0])

# %%
from sklearn.impute import SimpleImputer
model=SimpleImputer(missing_values=np.nan,strategy='mean')
df_model=model.fit_transform(df)

# %%
df_df=pd.DataFrame(data=df_model,columns=df.columns)
type(df_df)

# %%
#9、对名义型变量做独热编码
df_one=pd.get_dummies(df_df)#做独热编码

# %% [markdown]
# 10、将数据按照目标列和其他列进行拆分，并命名为x,y

# %%
x=[col for col in df_one.columns if col!='Cust_No' and col!='Target']
x

# %%
y=df['Target']
df_x=df_one[x]
y

# %%
#11、使用train_test_split对数据进行x,y拆分，拆分为训练集和测试集，拆分的测试集比例为0.3
from sklearn.model_selection import train_test_split
x_train,x_test,y_train,y_test=train_test_split(df_x,y,test_size=0.3)

# %% [markdown]
# 12、引入随机深林算法随机指定参数

# %%
#使用随机深林建模，训练，输出模型
from sklearn.ensemble import RandomForestClassifier
model_R=RandomForestClassifier().fit(x_train,y_train)#
#print(round(model_R.score(x_test,y_test),2))#取小数点后两位

# %%
#预测
df_p=model_R.predict(x_test)
df_p

# %%
#对模型进行评分
from sklearn.metrics import accuracy_score
print(accuracy_score(y_test,df_p))

# %%
from imblearn.over_sampling import SMOTE
x_train,y_train=SMOTE().fit_resample(x_train,y_train)
pd.Series(y_train).value_counts()

# %%
#再次训练随机深林模型
from sklearn.ensemble import RandomForestClassifier
model_R2=RandomForestClassifier().fit(x_train,y_train)

# %%
df_p2=model_R2.predict(x_test)
df_p

# %%
from sklearn.metrics import f1_score
print(f1_score(y_test,df_p2))

# %%






# %%
import pandas as pd
import numpy as np
df = pd.read_csv('F:/ICT2023/AI-LAB/data/titanic_train.csv')
df.head()

# %%
df.tail()

# %%
for x in df.columns:
    qssl = df[x].isnull().sum()#缺失值数量
    qszb = df[x].isnull().sum()/df.shape[0]*100
    print(x,'缺失值数量：',qssl,'缺失值占比：',qszb,'%')

# %%
df.dropna(inplace=True)

# %%
df.isnull().sum()

# %%
# 获取数值特征统计信息
df.describe()

# %%
# 绘制数据集中所有数值型特征的直方图
import matplotlib.pyplot as plt
df.hist(figsize=(12,8))
plt.show()

# %%
# 绘制数据集中所有分类型特征的条形图
import seaborn as sn
x = ['Sex','Embarked','Pclass','Survived']
for f in x:
    sn.countplot(x=f,data=df)
    plt.show()

# %%
# 使用pandas的corr()方法计算数据集中数值型特征之间的相关系数矩阵
df_corr = df.corr()
df_corr

# %%
# 绘制相关系数矩阵的热力图
import seaborn as sn
sn.heatmap(df_corr,annot=True)

# %%
# 选择两个数值型特征绘制散点图，并添加趋势线
sn.regplot(x='Age',y='Fare',data=df)
plt.show()

# %%
# 对所有分类型特征进行独热编码
from sklearn.preprocessing import OneHotEncoder
categorical_features = ['Sex','Embarked','Pclass']
encoder = OneHotEncoder()
encoder_features = encoder.fit_transform(df[categorical_features]).toarray()
encoder_features_df = pd.DataFrame(encoder_features, columns=encoder.get_feature_names())
df.drop(categorical_features, axis=1, inplace=True)
data = pd.concat([df, encoder_features_df], axis=1)
data.dropna(inplace=True)


# %%
df_one = pd.get_dummies(df)#对数据集做独热编码

# %%
# 将数据集划分为训练集和测试集，比例为3:1，使用随机种子为42
x=[col for col in df_one.columns if col!='Survived']
X = df[x]
Y=df_one['Survived']

# %%
from sklearn.model_selection import train_test_split
x_train,x_test,y_train,y_test=train_test_split(X,Y,test_size=0.3,random_state=42)

# %%
# 使用线性回归模型对数据集进行拟合，计算模型的R方值
from sklearn.linear_model import LinearRegression,LogisticRegression
from sklearn.metrics import r2_score,accuracy_score,f1_score
lr = LinearRegression()# 建模
lr.fit(x_train, y_train)# 使用模型训练数据
y_pred = lr.predict(x_test)# 对模型进行预测
r2 = r2_score(y_test, y_pred)# 模型评估
print('线性回归模型的R方值：',r2)

# %%
# 使用逻辑回归对数据集进行拟合，计算模型的准确率
logistic_regression = LogisticRegression()#建模
logistic_regression.fit(x_train,y_train)#进行模型训练
y_pred = logistic_regression.predict(x_test)#预测
accuracy = accuracy_score(y_test,y_pred)#模型评分
print('逻辑回归模型的准确率：',accuracy)

# %%
# 使用随机森林模型对数据集进行拟合，计算模型的准确率
from sklearn.ensemble import RandomForestClassifier
random_forest = RandomForestClassifier()
random_forest.fit(x_train,y_train)
y_pred = random_forest.predict(x_test)
accuracy = accuracy_score(y_test,y_pred)
print('随机森林模型的准确率：',accuracy)

# %%
# 使用accuracy_score进行评估
from sklearn.metrics import accuracy_score,f1_score
print('逻辑回归模型的准确率(使用accuracy_score进行评估)：',accuracy_score(y_test,y_pred))

# %%
# 使用网格搜索对随机森林模型进行调参，选择最优参数，过采样（SMOTE）
from sklearn.model_selection import GridSearchCV
param_grid = {'n_estimators':[10,20,30,40],'max_depth':[3,4,5,6]}
grid_search = GridSearchCV(random_forest,param_grid=param_grid,cv=10)
grid_search.fit(x_train,y_train)
print('最优参数：',grid_search.best_params_)

# %%
# 使用最优参数重新训练随机森林模型，并计算模型的准确率
random_forest = grid_search.best_estimator_
random_forest.fit(x_train,y_train)
y_pred = random_forest.predict(x_test)
accuracy = accuracy_score(y_test,y_pred)
print('优化后的随机森林模型的准确率：',accuracy)

# %%
# 对调优后的模型使用f1值评估
print('优化后的随机森林模型的f1值：',f1_score(y_test,y_pred,average='micro'))

# %%






import requests

def go(content,type=0):
    url = 'https://www.heyus.cn/api/translate'
    headers = {
        'Content-Type': 'application/json',
    }

    # content = '请问为什么1+1=2？'
    # data = {'type':'999','message':content}
    data = {'type':type,'message':content}

    response = requests.post(url, headers=headers, json=data)
    return response.text.replace('\\n','\n')


import tensorflow as tf
from tensorflow.python.keras.layers import Conv2D, MaxPooling2D, Flatten, Dense
from tensorflow.python.keras import regularizers

# Define parameters
batch_size = 128
epochs = 20
IMG_HEIGHT = 100
IMG_WIDTH = 100
l2_rate = 0.01

# rescale在这里把图片的像素值缩小了255倍，除以255
train_image_generator = tf.keras.preprocessing.image.ImageDataGenerator(
    rescale=1./255, validation_split=0.2)
# directory 目标路径
# batch_size  每次输入到模型中的样本数量
# shuffle 开始之前是否对数据进行随机打乱
# target_size 是指在读取图像时，将所有图像调整为固定大小的操作
# class_mode categorical代表分类编码的形式 还有binary二元分类 sparse稀疏分类问题
train_data_gen = train_image_generator.flow_from_directory(directory='flower_photos',
                                                           batch_size=batch_size,
                                                           shuffle=True,
                                                           target_size=(
                                                               IMG_HEIGHT, IMG_WIDTH),
                                                           class_mode='categorical',
                                                           subset='training')

validation_data_gen = train_image_generator.flow_from_directory(directory='flower_photos',
                                                                batch_size=batch_size,
                                                                shuffle=False,
                                                                target_size=(
                                                                    IMG_HEIGHT, IMG_WIDTH),
                                                                class_mode='categorical',
                                                                subset='validation')

# 获取训练数据集和验证数据集中类别标签名称及其对应的数字编码
class_names = train_data_gen.class_indices

# 将数字编码和标签名称反转，得到标签名称列表
# label_names = list(class_names.keys())
print('标签名称列表为：', class_names)

# 用于构建序列模型，由多个神经网络层按照一定的顺序依次组成
model = tf.keras.Sequential([
    # Convolutional Layers 卷积层  -> 作用：提取图像特征

    # Conv2D(32, (3,3)：创建一个包含32个卷积核、大小为3x3的二维卷积层。
    # padding='same'：每次卷积运算都会缩小空间，为了避免某次输出大小为1，即使用零填充到输入特征图外围，使得输出特征图和输入特征图的大小相同。
    # activation='relu'：设置激活函数为ReLU，ReLU函数可以有效地缓解梯度消失问题，并且在卷积神经网络中表现良好。
    # input_shape=(IMG_HEIGHT, IMG_WIDTH ,3)：指定输入的形状，即待处理图像的高度、宽度和通道数。这里输入的是RGB三通道的彩色图像，所以通道数为3。
    # kernel_regularizer=regularizers.l2(l2_rate)：设置L2正则化项，用于控制模型的复杂度，防止欠拟合
    Conv2D(32, (3, 3), padding='same', activation='relu', input_shape=(IMG_HEIGHT, IMG_WIDTH, 3),
           kernel_regularizer=regularizers.l2(l2_rate)),
    MaxPooling2D(pool_size=(2, 2)),
    Conv2D(64, (3, 3), padding='same', activation='relu',
           kernel_regularizer=regularizers.l2(l2_rate)),
    MaxPooling2D(pool_size=(2, 2)),
    Conv2D(128, (3, 3), padding='same', activation='relu',
           kernel_regularizer=regularizers.l2(l2_rate)),
    MaxPooling2D(pool_size=(2, 2)),
    Conv2D(128, (3, 3), padding='same', activation='relu',
           kernel_regularizer=regularizers.l2(l2_rate)),
    MaxPooling2D(pool_size=(2, 2)),

    # 压缩128通道到到一维
    Flatten(),

    # Dense layers
    Dense(1024, activation='relu', kernel_regularizer=regularizers.l2(l2_rate)),
    Dense(512, activation='relu', kernel_regularizer=regularizers.l2(l2_rate)),
    Dense(256, activation='relu', kernel_regularizer=regularizers.l2(l2_rate)),
    Dense(128, activation='relu', kernel_regularizer=regularizers.l2(l2_rate)),
    Dense(5, activation='softmax')
])

# Compile the model with categorical crossentropy loss and Adam optimizer
# adam随机梯度下降
# 多分类任务的交叉熵损失函数
model.compile(optimizer='adam',
              loss=tf.keras.losses.CategoricalCrossentropy(from_logits=True),
              metrics=['accuracy'])

# Fit the model to the data generator for given number of epochs
# 整除
history = model.fit(train_data_gen,
                    steps_per_epoch=train_data_gen.samples // batch_size,
                    epochs=epochs,
                    validation_data=validation_data_gen,
                    validation_steps=validation_data_gen.samples // batch_size)

# Save the trained model to a file
model.save('model/model_test.ckpt')



import tensorflow as tf
import numpy as np
from PIL import Image

# 加载模型
model = tf.keras.models.load_model('model/model_test.ckpt')

# 加载图片，并进行预处理
img = Image.open('flower_photos/roses/123128873_546b8b7355_n.jpg').resize((100, 100))
img_arr = np.array(img) / 255.0
img_arr = np.expand_dims(img_arr, axis=0)

# 使用模型进行预测
result = model.predict(img_arr)
label = np.argmax(result, axis=1)

# 输出预测结果
print('预测结果为：', label)



# %% [markdown]
# ## 8.银行客户聚类

# %%
import pandas as pd
import numpy as np

# %% [markdown]
# ### 1.导入数据，查看数据前五行

# %%
df = pd.read_csv('./bankmarketing.csv')
df.head()

# %% [markdown]
# ### 2.对数据进行缺失值处理

# %%
df.isnull().sum()

# %%
df.fillna(df.mode().iloc[0],inplace=True)

# %%
df.isnull().sum()

# %% [markdown]
# ### 3.对数据进行编码处理

# %%
df_oh = pd.get_dummies(df)
df_oh.head()

# %% [markdown]
# ### 4.对数据进行主成分分析降维，主成分对原数据解释比例不低于95%

# %%
from sklearn.decomposition import PCA

# %%
pca_model = PCA(n_components=0.95)
df_pca = pca_model.fit_transform(df_oh)
df_pca = pd.DataFrame(df_pca)
df_pca.head()

# %% [markdown]
# ### 5.对数据进行标准化

# %%
from sklearn.preprocessing import StandardScaler

# %%
scale = StandardScaler()
df_std = scale.fit_transform(df_pca)

# %% [markdown]
# ### 6.使用k-means算法进行建模，使用sse和轮廓系数对模型结果进行评估。对轮廓系数绘制折线图，根据绘图选出最优的k值

# %%
from sklearn.cluster import KMeans
from sklearn.metrics import silhouette_score
from matplotlib import pyplot as plt

# %%
k,silhouette = [],[]
for i in range(2,6):
    km_model = KMeans(n_clusters=i)
    km_model.fit(df_std)
    k.append(i)
    sil = silhouette_score(df_std,km_model.labels_)
    silhouette.append(sil)
    sse = km_model.inertia_
    print(f'k={i},silhouette={sil},sse={sse}')

# %%
plt.plot(k,silhouette)
plt.xlabel('k')
plt.ylabel('silhouette')
plt.show()

# %% [markdown]
# ### 7.使用Birch算法对用户业务流量进行聚类分析，并绘制轮廓系数的手肘图选出最优的k值

# %%
from sklearn.cluster import Birch

# %%
k,silhouette = [],[]
for i in range(2,8):
    birch_model = Birch(n_clusters=i)
    birch_model.fit(df_std)
    k.append(i)
    sil = silhouette_score(df_std,birch_model.labels_)
    silhouette.append(sil)
    sse = km_model.inertia_
    print(f'k={i},silhouette={sil},sse={sse}')

# %%
plt.plot(k,silhouette)
plt.xlabel('k')
plt.ylabel('silhouette')
plt.show()

# %% [markdown]
# ### 8.使用SpecteralClustering算法对用户业务流量进行聚类分析，并绘制轮廓系数的手肘图选出最优的k值

# %%
from sklearn.cluster import SpectralClustering

# %%
k,silhouette = [],[]
for i in range(2,6):
    spe_model = SpectralClustering(n_clusters=i)
    spe_model.fit(df_std)
    k.append(i)
    sil = silhouette_score(df_std,spe_model.labels_)
    silhouette.append(sil)
    see = spe_model.inertia_
#     print(f'k={i},silhouette={sil},sse={sse}')

# %%
plt.plot(k,silhouette)
plt.xlabel('k')
plt.ylabel('silhouette')
plt.show()

# %% [markdown]
# ### 9.根据前面三种算法的结果，选出最优的算法模型，利用最优的算法模型输出数据集的聚类标签，并将标签导入到数据集中

# %%
birch_model = Birch(n_clusters=2)
label = birch_model.fit_predict(df_std)
label

# %%
df_oh['label'] = label
df_oh.head()

# %% [markdown]
# ### 10.查看不同聚类结果下的特征均值，并绘制不同聚类结果下的特征均值折线图

# %%
df.columns

# %%
mean_dict = {'age':'mean','duration':'mean','campaign':'mean'}
data_label = df_oh.groupby('label').agg(mean_dict)
data_label

# %%
plt.figure(figsize=(20,10))
for i in data_label.index:    
    plt.plot(['age','duration','campaign'],data_label.iloc[i],'o-')
plt.show()

# %% [markdown]
# ### 11.绘制三维散点图查看各簇的数据在三维特征下的数据分布，并描述各个簇的特点

# %%
import mpl_toolkits.mplot3d

# %%
from mpl_toolkits.mplot3d import Axes3D 
colors = ['r','g'] 
markers = ['s', 'x'] 
fig = plt.figure(figsize=(10,8)) 
ax = Axes3D(fig) 
for i in range(0,2): 
    ax.scatter(df_oh['age'][df_oh['label']==i],
               df_oh['duration'][df_oh['label']==i],
               df_oh['campaign'][df_oh['label']==i],
               c=colors[i],marker=markers[i]) # 绘制三维散点图 
ax.view_init(elev=20,azim=30) 
ax.set_xlabel('age') 
ax.set_ylabel('duration') 
ax.set_zlabel('campaign') 
plt.show() 


